@@ -24,10 +24,8 @@
     [org.apache.maven.model.resolution ModelResolver]
     ;; maven-resolver-impl
     [org.eclipse.aether.internal.impl DefaultRemoteRepositoryManager]
-    ;; maven-resolver-spi
-    [org.eclipse.aether.spi.locator ServiceLocator]
     ;; maven-model
-    [org.apache.maven.model Resource]
+    [org.apache.maven.model Resource License]
     ;; maven-core
     [org.apache.maven.project ProjectModelResolver ProjectBuildingRequest$RepositoryMerging]
     ))
@@ -37,8 +35,8 @@
 (defn- model-resolver
   ^ModelResolver [{:keys [mvn/repos mvn/local-repo]} settings]
   (let [local-repo (or local-repo @maven/cached-local-repo)
-        locator ^ServiceLocator @maven/the-locator
-        system (maven/make-system)
+        locator (maven/make-locator)
+        system (maven/make-system locator)
         session (maven/make-session system settings local-repo)
         repo-mgr (doto (DefaultRemoteRepositoryManager.) (.initService locator))
         repos (maven/remote-repos repos settings)]
@@ -129,6 +127,20 @@
   [_lib {:keys [deps/root] :as _coord} _mf _config]
   (.getAbsolutePath (jio/file root "pom.xml")))
 
+(defmethod ext/license-info-mf :pom
+  [lib {:keys [deps/root] :as _coord} _mf config]
+  (let [pom (jio/file root "pom.xml")
+        model (read-model-file pom config)
+        licenses (.getLicenses model)
+        ^License license (when (and licenses (pos? (count licenses))) (first licenses))]
+    (when license
+      (let [name (.getName license)
+            url (.getUrl license)]
+        (when (or name url)
+          (cond-> {}
+            name (assoc :name name)
+            url (assoc :url url)))))))
+
 (defmethod ext/coord-usage :pom
   [lib {:keys [deps/root]} manifest-type config]
   ;; TBD
